import BigQueryFormatter from 'src/languages/bigquery.formatter';
import Db2Formatter from 'src/languages/db2.formatter';
import HiveFormatter from 'src/languages/hive.formatter';
import MariaDbFormatter from 'src/languages/mariadb.formatter';
import MySqlFormatter from 'src/languages/mysql.formatter';
import N1qlFormatter from 'src/languages/n1ql.formatter';
import PlSqlFormatter from 'src/languages/plsql.formatter';
import PostgreSqlFormatter from 'src/languages/postgresql.formatter';
import RedshiftFormatter from 'src/languages/redshift.formatter';
import SparkFormatter from 'src/languages/spark.formatter';
import SqliteFormatter from 'src/languages/sqlite.formatter';
import SqlFormatter from 'src/languages/sql.formatter';
import TSqlFormatter from 'src/languages/tsql.formatter';
import type { FormatOptions } from './types';
export declare const formatters: {
    bigquery: typeof BigQueryFormatter;
    db2: typeof Db2Formatter;
    hive: typeof HiveFormatter;
    mariadb: typeof MariaDbFormatter;
    mysql: typeof MySqlFormatter;
    n1ql: typeof N1qlFormatter;
    plsql: typeof PlSqlFormatter;
    postgresql: typeof PostgreSqlFormatter;
    redshift: typeof RedshiftFormatter;
    spark: typeof SparkFormatter;
    sql: typeof SqlFormatter;
    sqlite: typeof SqliteFormatter;
    tsql: typeof TSqlFormatter;
};
export declare type SqlLanguage = keyof typeof formatters;
export declare const supportedDialects: string[];
export declare type FormatFnOptions = FormatOptions & {
    language: SqlLanguage;
};
/**
 * Format whitespace in a query to make it easier to read.
 *
 * @param {string} query - input SQL query string
 * @param {FormatOptions} cfg Configuration options (see docs in README)
 * @return {string} formatted query
 */
export declare const format: (query: string, cfg?: Partial<FormatFnOptions>) => string;
export declare class ConfigError extends Error {
}
export declare type FormatFn = typeof format;
